@@ -32,6 +32,9 @@ struct XDMCPServerPrivate
     /* Port to listen on */
     guint port;
 
+    /* Address to listen on */
+    gchar *listen_address;
+
     /* Listening sockets */
     GSocket *socket, *socket6;
 
@@ -73,6 +76,22 @@ xdmcp_server_get_port (XDMCPServer *server)
     return server->priv->port;
 }
 
+void
+xdmcp_server_set_listen_address (XDMCPServer *server, const gchar *listen_address)
+{
+    g_return_if_fail (server != NULL);
+
+    g_free (server->priv->listen_address);
+    server->priv->listen_address = g_strdup (listen_address);
+}
+
+const gchar *
+xdmcp_server_get_listen_address (XDMCPServer *server)
+{
+    g_return_val_if_fail (server != NULL, NULL);
+    return server->priv->listen_address;
+}
+
 void
 xdmcp_server_set_hostname (XDMCPServer *server, const gchar *hostname)
 {
@@ -258,10 +277,76 @@ decode_key (const gchar *key, guint8 *data)
     }
 }
 
+static GInetAddress *
+connection_to_address (XDMCPConnection *connection)
+{
+    switch (connection->type)
+    {
+    case XAUTH_FAMILY_INTERNET:
+        if (connection->address.length == 4)
+            return g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV4);
+        else
+            return NULL;
+    case XAUTH_FAMILY_INTERNET6:
+        if (connection->address.length == 16)
+            return g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV6);
+        else
+            return NULL;
+    default:
+        return NULL;
+    }
+}
+
+static gssize
+find_address (GInetAddress **addresses, gsize length, GSocketFamily family)
+{
+    int i;
+
+    for (i = 0; i < length; i++)
+    {
+        GInetAddress *address = addresses[i];
+        if (address && g_inet_address_get_family (address) == family)
+            return i;
+    }
+
+    return -1;
+}
+
+static XDMCPConnection *
+choose_connection (XDMCPPacket *packet, GInetAddress *source_address)
+{
+    GInetAddress **addresses;
+    gsize addresses_length, i;
+    gssize index = -1;
+
+    addresses_length = packet->Request.n_connections;
+    addresses = malloc (sizeof (GInetAddress *) * addresses_length);
+    for (i = 0; i < addresses_length; i++)
+        addresses[i] = connection_to_address (&packet->Request.connections[i]);
+
+    /* Use the address the request came in on as this is the least likely to have firewall / routing issues */
+    for (i = 0; i < addresses_length && index < 0; i++)
+        if (g_inet_address_equal (source_address, addresses[i]))
+            index = i;
+
+    /* Otherwise try and find an address that matches the incoming type */
+    if (index < 0)
+        index = find_address (addresses, addresses_length, g_inet_address_get_family (source_address));
+
+    /* Otherwise use the first available */
+    if (index < 0 && addresses_length > 0)
+        index = 0;
+
+    for (i = 0; i < addresses_length; i++)
+        g_object_unref (addresses[i]);
+    g_free (addresses);
+
+    return &packet->Request.connections[index];
+}
+
 static void
 handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, XDMCPPacket *packet)
 {
-    int i;
     XDMCPPacket *response;
     XDMCPSession *session;
     guint8 *authentication_data = NULL;
@@ -273,48 +358,15 @@ handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, X
     guint8 *session_authorization_data = NULL;
     gsize session_authorization_data_length = 0;
     gchar **j;
-    guint16 family;
-    GInetAddress *x_server_address = NULL;
+    XDMCPConnection *connection;
     gchar *display_number;
     XdmAuthKeyRec rho;
 
-    /* Try and find an IPv6 address */
-    for (i = 0; i < packet->Request.n_connections; i++)
-    {
-        XDMCPConnection *connection = &packet->Request.connections[i];
-        if (connection->type == XAUTH_FAMILY_INTERNET6 && connection->address.length == 16)
-        {
-            family = connection->type;
-            x_server_address = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV6);
-
-            /* We can't use link-local addresses, as we need to know what interface it is on */
-            if (g_inet_address_get_is_link_local (x_server_address))
-            {
-                g_object_unref (x_server_address);
-                x_server_address = NULL;
-            }
-            else
-                break;
-        }
-    }
-
-    /* If no IPv6 address, then try and find an IPv4 one */
-    if (!x_server_address)
-    {
-        for (i = 0; i < packet->Request.n_connections; i++)
-        {
-            XDMCPConnection *connection = &packet->Request.connections[i];
-            if (connection->type == XAUTH_FAMILY_INTERNET && connection->address.length == 4)
-            {
-                family = connection->type;
-                x_server_address = g_inet_address_new_from_bytes (connection->address.data, G_SOCKET_FAMILY_IPV4);
-                break;
-            }
-        }
-    }
+    /* Choose an address to connect back on */
+    connection = choose_connection (packet, g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (address)));
 
     /* Decline if haven't got an address we can connect on */
-    if (!x_server_address)
+    if (!connection)
     {
         response = xdmcp_packet_alloc (XDMCP_Decline);
         response->Decline.status = g_strdup ("No valid address found");
@@ -430,34 +482,34 @@ handle_request (XDMCPServer *server, GSocket *socket, GSocketAddress *address, X
     }
 
     session = add_session (server);
-    session->priv->address = x_server_address;
+    session->priv->address = connection_to_address (connection);
     session->priv->display_number = packet->Request.display_number;
     display_number = g_strdup_printf ("%d", packet->Request.display_number);
 
     /* We need to check if this is the loopback address and set the authority
      * for a local connection if this is so as XCB treats "127.0.0.1" as local
      * always */
-    if (g_inet_address_get_is_loopback (x_server_address))
+    if (g_inet_address_get_is_loopback (session->priv->address))
     {
         gchar hostname[1024];
         gethostname (hostname, 1024);
 
         session->priv->authority = x_authority_new (XAUTH_FAMILY_LOCAL,
-                                              (guint8 *) hostname,
-                                              strlen (hostname),
-                                              display_number,
-                                              authorization_name,
-                                              session_authorization_data,
-                                              session_authorization_data_length);
+                                                    (guint8 *) hostname,
+                                                    strlen (hostname),
+                                                    display_number,
+                                                    authorization_name,
+                                                    session_authorization_data,
+                                                    session_authorization_data_length);
     }
     else
-        session->priv->authority = x_authority_new (family,
-                                              g_inet_address_to_bytes (G_INET_ADDRESS (x_server_address)),
-                                              g_inet_address_get_native_size (G_INET_ADDRESS (x_server_address)),
-                                              display_number,
-                                              authorization_name,
-                                              session_authorization_data,
-                                              session_authorization_data_length);
+        session->priv->authority = x_authority_new (connection->type,
+                                                    connection->address.data,
+                                                    connection->address.length,
+                                                    display_number,
+                                                    authorization_name,
+                                                    session_authorization_data,
+                                                    session_authorization_data_length);
     g_free (display_number);
 
     response = xdmcp_packet_alloc (XDMCP_Accept);
@@ -604,7 +656,7 @@ read_cb (GSocket *socket, GIOCondition condition, XDMCPServer *server)
 }
 
 static GSocket *
-open_udp_socket (GSocketFamily family, guint port, GError **error)
+open_udp_socket (GSocketFamily family, guint port, const gchar *listen_address, GError **error)
 {
     GSocket *socket;
     GSocketAddress *address;
@@ -614,7 +666,21 @@ open_udp_socket (GSocketFamily family, guint port, GError **error)
     if (!socket)
         return NULL;
 
-    address = g_inet_socket_address_new (g_inet_address_new_any (family), port);
+    if (listen_address) 
+    {
+        GList *addresses;
+
+        addresses = g_resolver_lookup_by_name (g_resolver_get_default (), listen_address, NULL, error);
+        if (!addresses)
+        {
+            g_object_unref (socket);
+            return NULL;
+        }
+        address = g_inet_socket_address_new (addresses->data, port);
+        g_resolver_free_addresses (addresses);
+    }
+    else
+        address = g_inet_socket_address_new (g_inet_address_new_any (family), port);
     result = g_socket_bind (socket, address, TRUE, error);
     if (!result)
     {
@@ -633,7 +699,7 @@ xdmcp_server_start (XDMCPServer *server)
 
     g_return_val_if_fail (server != NULL, FALSE);
 
-    server->priv->socket = open_udp_socket (G_SOCKET_FAMILY_IPV4, server->priv->port, &error);
+    server->priv->socket = open_udp_socket (G_SOCKET_FAMILY_IPV4, server->priv->port, server->priv->listen_address, &error);
     if (error)
         g_warning ("Failed to create IPv4 XDMCP socket: %s", error->message);
     g_clear_error (&error);
@@ -645,7 +711,7 @@ xdmcp_server_start (XDMCPServer *server)
         g_source_attach (source, NULL);
     }
 
-    server->priv->socket6 = open_udp_socket (G_SOCKET_FAMILY_IPV6, server->priv->port, &error);
+    server->priv->socket6 = open_udp_socket (G_SOCKET_FAMILY_IPV6, server->priv->port, server->priv->listen_address, &error);
     if (error)
         g_warning ("Failed to create IPv6 XDMCP socket: %s", error->message);
     g_clear_error (&error);
@@ -685,6 +751,7 @@ xdmcp_server_finalize (GObject *object)
         g_object_unref (self->priv->socket);
     if (self->priv->socket6)
         g_object_unref (self->priv->socket6);
+    g_free (self->priv->listen_address);
     g_free (self->priv->hostname);
     g_free (self->priv->status);
     g_free (self->priv->key);
@@ -703,7 +770,7 @@ xdmcp_server_class_init (XDMCPServerClass *klass)
     g_type_class_add_private (klass, sizeof (XDMCPServerPrivate));
 
     signals[NEW_SESSION] =
-        g_signal_new ("new-session",
+        g_signal_new (XDMCP_SERVER_SIGNAL_NEW_SESSION,
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (XDMCPServerClass, new_session),
