@@ -1,6 +1,8 @@
 #define _GNU_SOURCE
 #define __USE_GNU
 
+#include <config.h>
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -8,6 +10,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
 #include <pwd.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -24,6 +28,10 @@
 #include <xcb/xcb.h>
 #include <gio/gunixsocketaddress.h>
 
+#if HAVE_LIBAUDIT
+#include <libaudit.h>
+#endif
+
 #include "status.h"
 
 #define LOGIN_PROMPT "login:"
@@ -176,6 +184,14 @@ setresuid (uid_t ruid, uid_t uuid, uid_t suid)
 static gchar *
 redirect_path (const gchar *path)
 {
+    /* Hide Xmir for legacy tests */
+    if (g_str_has_suffix (path, "/Xmir"))
+    {
+        connect_status ();
+        if (g_key_file_get_boolean (config, "test-xmir", "hide", NULL))
+            return NULL;
+    }
+
     // Don't redirect if inside the running directory
     if (g_str_has_prefix (path, g_getenv ("LIGHTDM_TEST_ROOT")))
         return g_strdup (path);
@@ -336,6 +352,11 @@ access (const char *pathname, int mode)
 
     _access = (int (*)(const char *pathname, int mode)) dlsym (RTLD_NEXT, "access");
 
+    if (strcmp (pathname, "/dev/tty0") == 0)
+        return F_OK;
+    if (strcmp (pathname, "/sys/class/tty/tty0/active") == 0)
+        return F_OK;
+
     new_path = redirect_path (pathname);
     ret = _access (new_path, mode);
     g_free (new_path);
@@ -542,6 +563,188 @@ ioctl (int d, unsigned long request, ...)
     }
 }
 
+static void
+add_port_redirect (int requested_port, int redirected_port)
+{
+    GKeyFile *file;
+    gchar *path, *name, *data;
+
+    file = g_key_file_new ();
+    path = g_build_filename (g_getenv ("LIGHTDM_TEST_ROOT"), ".port-redirects", NULL);
+    g_key_file_load_from_file (file, path, G_KEY_FILE_NONE, NULL);
+
+    name = g_strdup_printf ("%d", requested_port);
+    g_key_file_set_integer (file, name, "redirected", redirected_port);
+    g_free (name);
+  
+    data = g_key_file_to_data (file, NULL, NULL);
+    g_file_set_contents (path, data, -1, NULL);
+    g_free (data);
+    g_free (path);
+
+    g_key_file_free (file);
+}
+
+static int
+find_port_redirect (int port)
+{
+    GKeyFile *file;
+    gchar *path, *name;
+    int redirected_port;
+
+    file = g_key_file_new ();
+    path = g_build_filename (g_getenv ("LIGHTDM_TEST_ROOT"), ".port-redirects", NULL);
+    g_key_file_load_from_file (file, path, G_KEY_FILE_NONE, NULL);
+    g_free (path);
+
+    name = g_strdup_printf ("%d", port);
+    redirected_port = g_key_file_get_integer (file, name, "redirected", NULL);
+    g_free (name);
+    g_key_file_free (file);
+
+    return redirected_port;
+}
+
+int
+bind (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
+{
+    int port = 0, redirected_port = 0;
+    int (*_bind) (int sockfd, const struct sockaddr *addr, socklen_t addrlen);
+    const struct sockaddr *modified_addr = addr;
+    struct sockaddr_in temp_addr;
+    struct sockaddr_in6 temp_addr6;
+    int retval;
+
+    _bind = (int (*)(int sockfd, const struct sockaddr *addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "bind");
+
+    switch (addr->sa_family)
+    {
+    case AF_INET:
+        port = ntohs (((const struct sockaddr_in *) addr)->sin_port);
+        redirected_port = find_port_redirect (port);
+        memcpy (&temp_addr, addr, sizeof (struct sockaddr_in));
+        modified_addr = (struct sockaddr *) &temp_addr;
+        if (redirected_port != 0)
+            temp_addr.sin_port = htons (redirected_port);
+        else
+            temp_addr.sin_port = 0;
+        break;
+    case AF_INET6:
+        port = ntohs (((const struct sockaddr_in6 *) addr)->sin6_port);
+        redirected_port = find_port_redirect (port);
+        memcpy (&temp_addr6, addr, sizeof (struct sockaddr_in6));
+        modified_addr = (struct sockaddr *) &temp_addr6;
+        if (redirected_port != 0)
+            temp_addr6.sin6_port = htons (redirected_port);
+        else
+            temp_addr6.sin6_port = 0;
+        break;
+    }
+
+    retval = _bind (sockfd, modified_addr, addrlen);
+
+    socklen_t temp_addr_len;
+    switch (addr->sa_family)
+    {
+    case AF_INET:
+        temp_addr_len = sizeof (temp_addr);
+        getsockname (sockfd, &temp_addr, &temp_addr_len);
+        if (redirected_port == 0)
+        {
+            redirected_port = ntohs (temp_addr.sin_port);
+            add_port_redirect (port, redirected_port);
+        }
+        break;
+    case AF_INET6:
+        temp_addr_len = sizeof (temp_addr6);
+        getsockname (sockfd, &temp_addr6, &temp_addr_len);
+        if (redirected_port == 0)
+        {
+            redirected_port = ntohs (temp_addr6.sin6_port);
+            add_port_redirect (port, redirected_port);
+        }
+        break;
+    }
+
+    return retval;
+}
+
+int
+connect (int sockfd, const struct sockaddr *addr, socklen_t addrlen)
+{
+    int port, redirected_port;
+    const struct sockaddr *modified_addr = addr;
+    struct sockaddr_in temp_addr;
+    struct sockaddr_in6 temp_addr6;
+    int (*_connect) (int sockfd, const struct sockaddr *addr, socklen_t addrlen);
+
+    _connect = (int (*)(int sockfd, const struct sockaddr *addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "connect");
+
+    switch (addr->sa_family)
+    {
+    case AF_INET:
+        port = ntohs (((const struct sockaddr_in *) addr)->sin_port);
+        redirected_port = find_port_redirect (port);
+        if (redirected_port != 0) 
+        {
+            memcpy (&temp_addr, addr, sizeof (struct sockaddr_in));
+            temp_addr.sin_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr;
+        }
+        break;
+    case AF_INET6:
+        port = ntohs (((const struct sockaddr_in6 *) addr)->sin6_port);
+        redirected_port = find_port_redirect (port);
+        if (redirected_port != 0) 
+        {
+            memcpy (&temp_addr6, addr, sizeof (struct sockaddr_in6));
+            temp_addr6.sin6_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr6;
+        }
+        break;
+    }
+
+    return _connect (sockfd, modified_addr, addrlen);
+}
+
+ssize_t
+sendto (int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen)
+{
+    int port, redirected_port;
+    const struct sockaddr *modified_addr = dest_addr;
+    struct sockaddr_in temp_addr;
+    struct sockaddr_in6 temp_addr6;
+    ssize_t (*_sendto) (int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen);
+
+    _sendto = (ssize_t (*)(int sockfd, const void *buf, size_t len, int flags, const struct sockaddr *dest_addr, socklen_t addrlen)) dlsym (RTLD_NEXT, "sendto");
+
+    switch (dest_addr->sa_family)
+    {
+    case AF_INET:
+        port = ntohs (((const struct sockaddr_in *) dest_addr)->sin_port);
+        redirected_port = find_port_redirect (port);
+        if (redirected_port != 0) 
+        {
+            memcpy (&temp_addr, dest_addr, sizeof (struct sockaddr_in));
+            temp_addr.sin_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr;
+        }
+        break;
+    case AF_INET6:
+        port = ntohs (((const struct sockaddr_in6 *) dest_addr)->sin6_port);
+        redirected_port = find_port_redirect (port);
+        if (redirected_port != 0) 
+        {
+            memcpy (&temp_addr6, dest_addr, sizeof (struct sockaddr_in6));
+            temp_addr6.sin6_port = htons (redirected_port);
+            modified_addr = (struct sockaddr *) &temp_addr6;
+        }
+        break;
+    }
+
+    return _sendto (sockfd, buf, len, flags, modified_addr, addrlen);
+}
+
 int
 close (int fd)
 {
@@ -1033,6 +1236,8 @@ pam_authenticate (pam_handle_t *pamh, int flags)
             result = pamh->conversation.conv (1, (const struct pam_message **) msg, &resp, pamh->conversation.appdata_ptr);
             free (msg[0]);
             free (msg);
+            if (result != PAM_SUCCESS)
+                return result;
 
             if (resp == NULL)
                 return PAM_CONV_ERR;
@@ -1737,3 +1942,48 @@ xcb_disconnect (xcb_connection_t *c)
         g_object_unref (c->socket);
     free (c);
 }
+
+#if HAVE_LIBAUDIT
+int
+audit_open (void)
+{
+    connect_status ();  
+    if (g_key_file_get_boolean (config, "test-audit-config", "check-events", NULL))
+        status_notify ("AUDIT OPEN");
+
+    return 1;
+}
+
+int
+audit_log_acct_message (int audit_fd, int type, const char *pgname,
+                        const char *op, const char *name, unsigned int id,
+                        const char *host, const char *addr, const char *tty, int result)
+{
+    gchar *type_string;
+
+    connect_status ();  
+    if (!g_key_file_get_boolean (config, "test-audit-config", "check-events", NULL))
+        return 1;
+
+    switch (type)
+    {
+    case AUDIT_USER_LOGIN:
+        type_string = g_strdup ("USER_LOGIN");
+        break;      
+    case AUDIT_USER_LOGOUT:
+        type_string = g_strdup ("USER_LOGOUT");
+        break;
+    default:
+        type_string = g_strdup_printf ("%d", type);
+        break;
+    }
+  
+    status_notify ("AUDIT LOG-ACCT TYPE=%s PGNAME=%s OP=%s NAME=%s ID=%u HOST=%s ADDR=%s TTY=%s RESULT=%d",
+                   type_string, pgname ? pgname : "", op ? op : "", name ? name : "", id, host ? host : "", addr ? addr : "", tty ? tty : "", result);
+
+    g_free (type_string);
+
+    return 1;
+}
+
+#endif
